searchData={"items":[{"type":"module","title":"AtuinStand","doc":"`AtuinStand` is a generic tree data structure for Elixir. Each node in the tree\ncan have user-defined data associated with it, and can have any number of children.\n\nAtuinStand is an implementation of the\n[`atuin-stand` project](https://github.com/atuinsh/atuin-stand).","ref":"AtuinStand.html"},{"type":"module","title":"API overview - AtuinStand","doc":"The AtuinStand API is split into two main parts:\n\n* `AtuinStand.Tree` - functions that operate on the entire tree\n* `AtuinStand.Node` - functions that operate on a single node","ref":"AtuinStand.html#module-api-overview"},{"type":"module","title":"Creating a tree - AtuinStand","doc":"```elixir\ntree = AtuinStand.Tree.new()\n```\n\n`AtuinStand.Tree.new/0` calls `Agent.start_link/3` to create a new process to manage the state.\nIf creating the process fails, it returns an error tuple in the same form as is returned\nfrom `Agent.start_link/3`.","ref":"AtuinStand.html#module-creating-a-tree"},{"type":"module","title":"Importing and exporting a tree - AtuinStand","doc":"You can export the tree as a map with `AtuinStand.Tree.export/1`.\n\n```elixir\ntree_data = AtuinStand.Tree.export(tree)\n```\n\nTo import existing data to a new tree instance, use `AtuinStand.Tree.import/1`.\n\n```elixir\ntree = AtuinStand.Tree.import(tree_data)\n```\n\nThe data exported by `AtuinStand.Tree.export/1` can be safely serialized to JSON.\nKeep in mind that any keys in user-defined data will be serialized as strings,\neven if they were originally created as atoms.","ref":"AtuinStand.html#module-importing-and-exporting-a-tree"},{"type":"module","title":"Getting the root node - AtuinStand","doc":"The root node is a special node that is always present in the tree. It is the ultimate\nancestor of all other nodes. Note that it cannot have associated data, and cannot be\nmoved in the tree or deleted.\n\n```elixir\nroot = AtuinStand.Tree.root(tree)\n```","ref":"AtuinStand.html#module-getting-the-root-node"},{"type":"module","title":"Creating a new child node - AtuinStand","doc":"To create a new node, call `AtuinStand.Node.create_child/2` with the parent node and the ID of\nthe new node. Note that all node IDs must be strings. The one exception is the root node,\nwhich has the ID `:root`.\n\n```elixir\nchild = AtuinStand.Node.create_child(root, \"child\")\nchild.id\n# => \"child\"\n```\n\nNodes in the tree are ordered, and by default, a newly created child node is placed at the end\nof its parent's children. If you'd like to place the child at a specific index within its siblings,\npass the index as the third argument to `AtuinStand.Node.create_child/2`.\n\n```elixir\nchild = AtuinStand.Node.create_child(root, \"child1\")\nchild = AtuinStand.Node.create_child(root, \"child2\", 0)\nAtuinStand.Node.children(root)\n# => [child2, child1]\n```","ref":"AtuinStand.html#module-creating-a-new-child-node"},{"type":"module","title":"Querying nodes - AtuinStand","doc":"You can check if a node exists with `AtuinStand.Tree.has_node/2`.\n\n```elixir\nAtuinStand.Tree.has_node(tree, \"child\")\n# => true\n```\n\nYou can get a node by ID with `AtuinStand.Tree.node/2`.\n\n```elixir\nnode = AtuinStand.Tree.node(tree, \"child\")\n```\n\nYou can fetch all of the external nodes (leaves) or internal nodes (branches)\nwith `AtuinStand.Tree.external_nodes/1` and `AtuinStand.Tree.internal_nodes/1`, respectively.\nThese are aliased as `AtuinStand.Tree.leaves/1` and `AtuinStand.Tree.branches/1`.\n\n```elixir\nleaves = AtuinStand.Tree.external_nodes(tree)\nbranches = AtuinStand.Tree.internal_nodes(tree)\n```\n\nFinally, you can fetch all the nodes in a tree with `AtuinStand.Tree.nodes/2`.\n\n```elixir\nnodes = AtuinStand.Tree.nodes(tree, :dfs) # or :bfs\n```","ref":"AtuinStand.html#module-querying-nodes"},{"type":"module","title":"Traversing the tree - AtuinStand","doc":"There are several functions for traversing the tree from a given node:\n\n* [`AtuinStand.Node.parent(node)`](`AtuinStand.Node.parent/1`)\n* [`AtuinStand.Node.children(node)`](`AtuinStand.Node.children/1`)\n* [`AtuinStand.Node.siblings(node)`](`AtuinStand.Node.siblings/1`)\n* [`AtuinStand.Node.ancestors(node)`](`AtuinStand.Node.ancestors/1`)\n* [`AtuinStand.Node.descendants(node, order)`](`AtuinStand.Node.descendants/2`)\n\nSee the `AtuinStand.Node` module for more information on these functions.","ref":"AtuinStand.html#module-traversing-the-tree"},{"type":"module","title":"Manipulating nodes - AtuinStand","doc":"","ref":"AtuinStand.html#module-manipulating-nodes"},{"type":"module","title":"Moving nodes - AtuinStand","doc":"You can move a node to a new parent with `AtuinStand.Node.move_to/2`. By default,\nthe node is moved to the end of the new parent's children. If you'd like to place the\nnode at a specific index within its new siblings, pass the index as the second argument.\n\n```elixir\nroot = AtuinStand.Tree.root(tree)\nchild1 = AtuinStand.Node.create_child(root, \"child1\")\nchild2 = AtuinStand.Node.create_child(root, \"child2\")\nAtuinStand.Node.move_to(child2, child1)\nAtuinStand.Node.children(root)\n# => [child1]\nAtuinStand.Node.children(child1)\n# => [child2]\n```\n\nYou can also move a node to be directly before or after another node using\n`AtuinStand.Node.move_before/2` and `AtuinStand.Node.move_after/2`, and you can reposition\na node within its siblings using `AtuinStand.Node.reposition/2`.","ref":"AtuinStand.html#module-moving-nodes"},{"type":"module","title":"Deleting nodes - AtuinStand","doc":"To delete a node, you must specifcy what to do with that node's children, if it has\nany. The options are:\n\n* `:refuse` - return an error if the node being deleted has children\n* `:cascade` - recursively delete the node and all of its children\n* `:reattach` - move the node's children to the node's parent before deleting it\n\n```elixir\ntree = AtuinStand.Tree.new()\nroot = AtuinStand.Tree.root(tree)\nchild1 = AtuinStand.Node.create_child(root, \"child1\")\nchild2 = AtuinStand.Node.create_child(child1, \"child2\")\nAtuinStand.Node.delete(child1, :decline)\n# => {:error, :has_children}\nAtuinStand.Node.delete(child1, :cascade)\nAtuinStand.Tree.nodes(tree)\n# => [root]\n```","ref":"AtuinStand.html#module-deleting-nodes"},{"type":"module","title":"Associated data - AtuinStand","doc":"You can set and get user-defined data with `AtuinStand.Node.set_data/2` and\n`AtuinStand.Node.get_data/1`. To remain compatible with other `atuin-stand`\nimplementations, the data must be a JSON-serializable map.\n\n```elixir\nAtuinStand.Node.set_data(node, %{\"name\" => \"Node 1\"})\nAtuinStand.Node.get_data(node)\n# => %{\"name\" => \"Node 1\"}\n```","ref":"AtuinStand.html#module-associated-data"},{"type":"module","title":"AtuinStand.Node","doc":"A node in an `AtuinStand.Tree`.\n\nYou can access the node's ID via the `id` property, and the tree it belongs to\nvia the `tree` property.\n\nSince `Node` structs only hold a reference to their containing tree, nodes\nmight be invalidated if the tree is manipulated such that the node is removed.\nIn this case, the `Node` functions will return `{:error, :not_found}`.\n\nFor a more detailed overview of the API, see `AtuinStand`.","ref":"AtuinStand.Node.html"},{"type":"module","title":"Examples - AtuinStand.Node","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    %AtuinStand.Node{id: :root, tree: tree}\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    %AtuinStand.Node{id: \"node1\", tree: tree}","ref":"AtuinStand.Node.html#module-examples"},{"type":"function","title":"AtuinStand.Node.ancestors/1","doc":"Returns a list of all ancestors of the given node, starting at the node's parent and\nending at the root node (inclusive).\n\nReturns `{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#ancestors/1"},{"type":"function","title":"Examples - AtuinStand.Node.ancestors/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(node1, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(node2, \"node3\")\n    iex> AtuinStand.Node.ancestors(node3)\n    [node2, node1, root]","ref":"AtuinStand.Node.html#ancestors/1-examples"},{"type":"function","title":"AtuinStand.Node.children/1","doc":"Returns a list of all children of the given node.\n\nReturns `{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#children/1"},{"type":"function","title":"Examples - AtuinStand.Node.children/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> AtuinStand.Node.children(root)\n    [node1, node2]\n    iex> AtuinStand.Node.children(node1)\n    []","ref":"AtuinStand.Node.html#children/1-examples"},{"type":"function","title":"AtuinStand.Node.create_child/2","doc":"Creates a new child node with the given ID.\n\nUser-created nodes must have unique, string IDs. Returns `{:error, :duplicate_id}`\nif a node with the given ID already exists in the tree. Returns `{:error, :not_found}`\nif the parent node is not found in the tree.","ref":"AtuinStand.Node.html#create_child/2"},{"type":"function","title":"Examples - AtuinStand.Node.create_child/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    %AtuinStand.Node{id: \"node1\", tree: tree}\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    {:error, :duplicate_id}","ref":"AtuinStand.Node.html#create_child/2-examples"},{"type":"function","title":"AtuinStand.Node.delete/2","doc":"Deletes the node from the tree.\n\nReturns `{:error, :invalid_operation}` if the node is the root node. Returns\n`{:error, :not_found}` if the node is not found in the tree.\n\nProvide a `strategy` to specify what to do with the node's children:\n\n* `:refuse` - return `{:error, :has_children}` if the node being deleted has children\n* `:cascade` - recursively delete the node and all of its children\n* `:reattach` - move the node's children to the node's parent before deleting it","ref":"AtuinStand.Node.html#delete/2"},{"type":"function","title":"Examples - AtuinStand.Node.delete/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(node1, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(node2, \"node3\")\n    iex> AtuinStand.Node.delete(node1, :refuse)\n    {:error, :has_children}\n    iex> AtuinStand.Node.delete(node1, :reattach)\n    iex> AtuinStand.Node.descendants(root)\n    [node2, node3]\n    iex> AtuinStand.Node.delete(node2, :cascade)\n    iex> AtuinStand.Node.descendants(root)\n    []","ref":"AtuinStand.Node.html#delete/2-examples"},{"type":"function","title":"AtuinStand.Node.depth/1","doc":"Returns the depth of the given node.\n\nFor any node, the depth is the number of edges on the path to the root node.\nThe root node has a depth of 0, and every other node has a depth of 1 + its parent's depth.\n\nEquivalent to `length(AtuinStand.Node.ancestors(node))`.\n\nReturns `{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#depth/1"},{"type":"function","title":"Examples - AtuinStand.Node.depth/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.depth(root)\n    0\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Node.depth(node1)\n    1\n    iex> node2 = AtuinStand.Node.create_child(node1, \"node2\")\n    iex> AtuinStand.Node.depth(node2)\n    2","ref":"AtuinStand.Node.html#depth/1-examples"},{"type":"function","title":"AtuinStand.Node.descendants/2","doc":"Returns a list of all descendants of the given node.\n\nProvide `:dfs` or `:bfs` as an optional argument to return the results in\ndepth-first or breadth-first order, respectively. Defaults to `:dfs`.\n\nReturns `{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#descendants/2"},{"type":"function","title":"Examples - AtuinStand.Node.descendants/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(node1, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(node2, \"node3\")\n    iex> node4 = AtuinStand.Node.create_child(root, \"node4\")\n    iex> AtuinStand.Node.descendants(node1, :dfs)\n    [node2, node3]\n    iex> AtuinStand.Node.descendants(root, :bfs)\n    [node1, node4, node2, node3]","ref":"AtuinStand.Node.html#descendants/2-examples"},{"type":"function","title":"AtuinStand.Node.get_data/1","doc":"Returns the user-defined data associated with the node.\n\nIf the node is not found, returns `{:error, :not_found}`.","ref":"AtuinStand.Node.html#get_data/1"},{"type":"function","title":"Examples - AtuinStand.Node.get_data/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Tree.node(tree, \"node1\")\n    ...> |> AtuinStand.Node.set_data(%{\"name\" => \"Node 1\"})\n    ...> |> AtuinStand.Node.get_data()\n    %{\"name\" => \"Node 1\"}","ref":"AtuinStand.Node.html#get_data/1-examples"},{"type":"function","title":"AtuinStand.Node.move_after/2","doc":"Moves the node after the given node.\n\nReturns `{:error, :invalid_operation}` if the node is the root node or if the move would create\na cycle in the tree. Returns `{:error, :not_found}` if either node is not found in the tree.","ref":"AtuinStand.Node.html#move_after/2"},{"type":"function","title":"Examples - AtuinStand.Node.move_after/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> AtuinStand.Node.move_after(node1, node3)\n    iex> AtuinStand.Node.children(root)\n    [node2, node3, node1]","ref":"AtuinStand.Node.html#move_after/2-examples"},{"type":"function","title":"AtuinStand.Node.move_before/2","doc":"Moves the node before the given node.\n\nReturns `{:error, :invalid_operation}` if the node is the root node or if the move would create\na cycle in the tree. Returns `{:error, :not_found}` if either node is not found in the tree.","ref":"AtuinStand.Node.html#move_before/2"},{"type":"function","title":"Examples - AtuinStand.Node.move_before/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> AtuinStand.Node.move_before(node3, node1)\n    iex> AtuinStand.Node.children(root)\n    [node3, node1, node2]","ref":"AtuinStand.Node.html#move_before/2-examples"},{"type":"function","title":"AtuinStand.Node.move_to/3","doc":"Moves the node to a new parent node.\n\nReturns `{:error, :invalid_operation}` if the node is the root node or if the move\nwould create a cycle in the tree. Returns `{:error, :not_found}` if the either node\nis not found in the tree.\n\nProvide an optional `index` to specify the position of the node in the new parent's\nchild list. The node will be inserted at the end if no index is provided.","ref":"AtuinStand.Node.html#move_to/3"},{"type":"function","title":"Examples - AtuinStand.Node.move_to/3","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(node1, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(node2, \"node3\")\n    iex> AtuinStand.Node.move_to(node1, node3)\n    {:error, :invalid_operation}\n    iex> AtuinStand.Node.move_to(node2, root)\n    iex> AtuinStand.Node.children(root)\n    [node1, node2]\n    iex> AtuinStand.Node.move_to(node3, root, 1)\n    iex> AtuinStand.Node.children(root)\n    [node1, node3, node2]\n    iex> AtuinStand.Node.move_to(node2, root, 1)\n    iex> AtuinStand.Node.children(root)\n    [node1, node2, node3]","ref":"AtuinStand.Node.html#move_to/3-examples"},{"type":"function","title":"AtuinStand.Node.parent/1","doc":"Returns the parent of the given node.\n\nReturns `{:error, :not_found}` if the node is not found in the tree.\nReturns `{:error, :invalid_node}` if the node is the root node.","ref":"AtuinStand.Node.html#parent/1"},{"type":"function","title":"Examples - AtuinStand.Node.parent/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.parent(root)\n    {:error, :invalid_node}\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Node.parent(node1)\n    %AtuinStand.Node{id: :root, tree: tree}\n    iex> fake_node = %AtuinStand.Node{id: \"fake\", tree: tree}\n    iex> AtuinStand.Node.parent(fake_node)\n    {:error, :not_found}","ref":"AtuinStand.Node.html#parent/1-examples"},{"type":"function","title":"AtuinStand.Node.reposition/2","doc":"Moves the node to a new position amongst its siblings.\n\nReturns `{:error, :invalid_operation}` if the node is the root node. Returns\n`{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#reposition/2"},{"type":"function","title":"Examples - AtuinStand.Node.reposition/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> AtuinStand.Node.reposition(node2, 0)\n    iex> AtuinStand.Node.children(root)\n    [node2, node1, node3]\n    iex> AtuinStand.Node.reposition(node2, 2)\n    iex> AtuinStand.Node.children(root)\n    [node1, node3, node2]","ref":"AtuinStand.Node.html#reposition/2-examples"},{"type":"function","title":"AtuinStand.Node.set_data/2","doc":"Sets the user-defined data associated with the node. Returns the node.\n\nThe data must be a map, otherwise returns `{:error, :invalid_data}`. When the\ntree is serialized to JSON, the data is serialized as well, so any atom keys\nwill be converted to strings.\n\nIf the node is not found, returns `{:error, :not_found}`.","ref":"AtuinStand.Node.html#set_data/2"},{"type":"function","title":"Examples - AtuinStand.Node.set_data/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Tree.node(tree, \"node1\")\n    ...> |> AtuinStand.Node.set_data(%{\"name\" => \"Node 1\"})\n    ...> |> AtuinStand.Node.get_data()\n    %{\"name\" => \"Node 1\"}","ref":"AtuinStand.Node.html#set_data/2-examples"},{"type":"function","title":"AtuinStand.Node.siblings/1","doc":"Returns a list of all siblings (other nodes with the same parent) of the given node.\n\nReturns `{:error, :not_found}` if the node is not found in the tree.","ref":"AtuinStand.Node.html#siblings/1"},{"type":"function","title":"Examples - AtuinStand.Node.siblings/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> AtuinStand.Node.siblings(node1)\n    [node2, node3]","ref":"AtuinStand.Node.html#siblings/1-examples"},{"type":"type","title":"AtuinStand.Node.t/0","doc":"","ref":"AtuinStand.Node.html#t:t/0"},{"type":"module","title":"AtuinStand.Tree","doc":"A generic, ordered tree.\n\nFor a more detailed overview of the API, see `AtuinStand`.","ref":"AtuinStand.Tree.html"},{"type":"function","title":"AtuinStand.Tree.branches/1","doc":"An alias for `internal_nodes/1`.","ref":"AtuinStand.Tree.html#branches/1"},{"type":"function","title":"AtuinStand.Tree.destroy/1","doc":"Shuts down the tree and frees up its resources.\n\nReturns `:ok` if the tree was successfully stopped, or `{:error, reason}` if it was not.\nSee `Agent.stop/1` for more information.\n\nNote that all nodes associated with the tree are invalid once the tree is destroyed.","ref":"AtuinStand.Tree.html#destroy/1"},{"type":"function","title":"AtuinStand.Tree.export/1","doc":"Exports the tree as an Elixir map.\n\nTo be compatible with other `atuin-stand` implementations, any user-defined data stored\nin the tree should use string keys.","ref":"AtuinStand.Tree.html#export/1"},{"type":"function","title":"AtuinStand.Tree.external_nodes/2","doc":"Returns a list of all external nodes (nodes with no children) in the tree, including the\nroot node (if applicable).\n\nProvide `:dfs` or `:bfs` as an optional argument to return the results in depth-first\nor breadth-first order, respectively. Defaults to `:dfs`.","ref":"AtuinStand.Tree.html#external_nodes/2"},{"type":"function","title":"Examples - AtuinStand.Tree.external_nodes/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Tree.external_nodes(tree)\n    [root]\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> node4 = AtuinStand.Node.create_child(node2, \"node4\")\n    iex> AtuinStand.Tree.external_nodes(tree, :dfs)\n    [node1, node4, node3]\n    iex> AtuinStand.Tree.external_nodes(tree, :bfs)\n    [node1, node3, node4]","ref":"AtuinStand.Tree.html#external_nodes/2-examples"},{"type":"function","title":"AtuinStand.Tree.has_node/2","doc":"Checks if a node with the given ID exists in the tree.\n\nThe only atom for which this function returns `true` is `:root`.","ref":"AtuinStand.Tree.html#has_node/2"},{"type":"function","title":"Examples - AtuinStand.Tree.has_node/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> AtuinStand.Tree.has_node(tree, \"node1\")\n    false\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Tree.has_node(tree, \"node1\")\n    true","ref":"AtuinStand.Tree.html#has_node/2-examples"},{"type":"function","title":"AtuinStand.Tree.import/1","doc":"Imports a tree from an Elixir map generated with `export/1`, or exported from\nanother `atuin-stand` implementation.\n\nCalls `Agent.start_link/3` to create a new process to manage the tree's state,\nand returns the tree, or an error tuple if `Agent.start_link/3` fails,\nsimilar to `new/0`.","ref":"AtuinStand.Tree.html#import/1"},{"type":"function","title":"Examples - AtuinStand.Tree.import/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> child = AtuinStand.Node.create_child(root, \"child\")\n    iex> AtuinStand.Node.set_data(child, %{\"name\" => \"Child\"})\n    iex> tree_data = AtuinStand.Tree.export(tree)\n    iex> AtuinStand.Tree.destroy(tree)\n    iex> tree = AtuinStand.Tree.import(tree_data)\n    iex> child = AtuinStand.Tree.node(tree, \"child\")\n    %AtuinStand.Node{id: \"child\", tree: tree}\n    iex> AtuinStand.Node.get_data(child)\n    %{\"name\" => \"Child\"}","ref":"AtuinStand.Tree.html#import/1-examples"},{"type":"function","title":"AtuinStand.Tree.internal_nodes/2","doc":"Returns a list of all internal nodes (nodes with children) in the tree, including the\nroot node (if applicable).\n\nProvide `:dfs` or `:bfs` as an optional argument to return the results in depth-first\nor breadth-first order, respectively. Defaults to `:dfs`.","ref":"AtuinStand.Tree.html#internal_nodes/2"},{"type":"function","title":"Examples - AtuinStand.Tree.internal_nodes/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(node1, \"node3\")\n    iex> _ = AtuinStand.Node.create_child(node2, \"node4\")\n    iex> _ = AtuinStand.Node.create_child(node3, \"node5\")\n    iex> AtuinStand.Tree.internal_nodes(tree, :dfs)\n    [root, node1, node3, node2]\n    iex> AtuinStand.Tree.internal_nodes(tree, :bfs)\n    [root, node1, node2, node3]","ref":"AtuinStand.Tree.html#internal_nodes/2-examples"},{"type":"function","title":"AtuinStand.Tree.leaves/1","doc":"An alias for `external_nodes/1`.","ref":"AtuinStand.Tree.html#leaves/1"},{"type":"function","title":"AtuinStand.Tree.new/0","doc":"Creates a new tree. Starts a new process linked to the current process\nto manage the tree's state.\n\nReturns the tree, or an error tuple if `Agent.start_link/3` fails.","ref":"AtuinStand.Tree.html#new/0"},{"type":"function","title":"AtuinStand.Tree.node/2","doc":"Returns the node with the given ID.\n\nReturns `{:error, :not_found}` if the node does not exist.","ref":"AtuinStand.Tree.html#node/2"},{"type":"function","title":"Examples - AtuinStand.Tree.node/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    iex> node1 = AtuinStand.Tree.node(tree, \"node1\")\n    iex> node1.id\n    \"node1\"","ref":"AtuinStand.Tree.html#node/2-examples"},{"type":"function","title":"AtuinStand.Tree.nodes/2","doc":"Returns a list of all nodes in the tree.\n\nProvide `:dfs` or `:bfs` as an optional argument to return the results in\ndepth-first or breadth-first order, respectively. Defaults to `:dfs`.","ref":"AtuinStand.Tree.html#nodes/2"},{"type":"function","title":"Examples - AtuinStand.Tree.nodes/2","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> node1 = AtuinStand.Node.create_child(root, \"node1\")\n    iex> node2 = AtuinStand.Node.create_child(root, \"node2\")\n    iex> node3 = AtuinStand.Node.create_child(root, \"node3\")\n    iex> node4 = AtuinStand.Node.create_child(node2, \"node4\")\n    iex> AtuinStand.Tree.nodes(tree, :dfs)\n    [root, node1, node2, node4, node3]\n    iex> AtuinStand.Tree.nodes(tree, :bfs)\n    [root, node1, node2, node3, node4]","ref":"AtuinStand.Tree.html#nodes/2-examples"},{"type":"function","title":"AtuinStand.Tree.root/1","doc":"Returns the root node of the tree.","ref":"AtuinStand.Tree.html#root/1"},{"type":"function","title":"Examples - AtuinStand.Tree.root/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> root.id\n    :root","ref":"AtuinStand.Tree.html#root/1-examples"},{"type":"function","title":"AtuinStand.Tree.size/1","doc":"Returns the number of nodes in the tree, including the root node.","ref":"AtuinStand.Tree.html#size/1"},{"type":"function","title":"Examples - AtuinStand.Tree.size/1","doc":"iex> tree = AtuinStand.Tree.new()\n    iex> AtuinStand.Tree.size(tree)\n    1\n    iex> root = AtuinStand.Tree.root(tree)\n    iex> AtuinStand.Node.create_child(root, \"node1\")\n    iex> AtuinStand.Tree.size(tree)\n    2\n    iex> AtuinStand.Node.create_child(root, \"node2\")\n    iex> AtuinStand.Tree.size(tree)\n    3","ref":"AtuinStand.Tree.html#size/1-examples"},{"type":"type","title":"AtuinStand.Tree.t/0","doc":"","ref":"AtuinStand.Tree.html#t:t/0"},{"type":"extras","title":"AtuinStand","doc":"# AtuinStand: An Elixir `atuin-stand` implementation\n\nAtuinStand is the Elixir implementation of the `atuin-stand` project. It implements a generic, ordered tree, designed for easy interop with the other `atuin-stand` implementations. See [the `atuin-stand` project](https://github.com/atuinsh/atuin-stand) for more information.","ref":"readme.html"},{"type":"extras","title":"Installation - AtuinStand","doc":"The package can be installed by adding `atuin_stand` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:atuin_stand, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - AtuinStand","doc":"The docs can be found at  .","ref":"readme.html#documentation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}